## Instruction
You are given a question, an answer, and one sentence extracted from the answer. 
Please break down the sentence into one or more fully understandable statements while also ensuring no pronouns are used in each statement.
Only use the given sentence extracted from the answer while breaking the sentece down into statements. Do not use the question or any other part of the answer.
The output should be a well-formatted JSON instance that conforms to the JSON schema below.

{{ output_model.model_json_schema() }}

## Examples
{% for example in examples %}
Question: {{ example.question }}
Answer: {{ example.answer }}
Sentence: {{ example.sentence }}
Analysis: {{ example.model_dump()['output'] }}

{% endfor %}


## Task
Question: {{ task.question }}
Answer: {{ task.answer }}
Sentence: {{ task.sentence }}
Analysis:
