## Instruction
Given question, answer and context verify if the context was useful in arriving at the given answer. 
Give verdict as "1" if useful and "0" if not.

The output should be a well-formatted JSON instance that conforms to the JSON schema below.

{{ output_model.model_json_schema() }}

## Examples
{% for example in examples %}
Question: {{ example.question }}
Context: {{ example.context }}
Answer: {{example.answer}}
Verification: {{ example.model_dump()['verification'] }}

{% endfor %}

## Task
Question: {{ task.question }}
Context: 
{% for document in documents %}
{{ document.content }}
{% endfor %}
Answer: {{task.answer}}
Verification:
