s## Instruction
Given a context, and an answer, analyze each sentence in the answer and classify if the sentence can be attributed to the given context or not. 
If the answer is only a word or a few words, consider the entire answer as one sentence. Do not use context elements as answer sentences.
ONLY use the answer while classifying and return it as it is. 
Provide the answer sentence, a rationale for your decision and the final classification label. 
Use only 1 (Yes) or 0 (No) as a binary classification.

The output should be a well-formatted JSON instance that conforms to the JSON schema below.

{{ output_model.model_json_schema() }}

## Examples
{% for example in examples %}
Question: "{{ example.question }}"
Context: "{{ example.context }}"
Answer: "{{ example.answer }}"
Classification: {{ example.model_dump()['classification'] }}

{% endfor %}

## Task
Question: "{{ task.question }}"
Context: 
{% for document in documents %}
{{ document.content }}
{% endfor %}
Answer: "{{ task.answer }}"
Classification: