## Instruction
Your task is to judge the faithfulness of a series of statements based on a given context. 
For each statement you must return verdict as 1 if the statement can be directly inferred based on the context or 0 if the statement can not be directly inferred based on the context.

The output should be a well-formatted JSON instance that conforms to the JSON schema below.

{{ output_model.model_json_schema() }}

## Examples
{% for example in examples %}
Context: {{ example.context }}
Statements: {{ example.statements }}
Answer: {{ example.model_dump()['output'] }}

{% endfor %}


## Task
Context: {{ task.context }}
Statements: {{ task.statements }}
Answer:
